# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014 Bryant E. McDonnell
#
# Licensed under the terms of the BSD2 License
# See LICENSE.txt for details
# -----------------------------------------------------------------------------
"""Base class for a SWMM Simulation."""

# Local imports
from pyswmm.swmm5 import PySWMM
from pyswmm.toolkitapi import SimulationTime, SimulationUnits


class Simulation(object):
    """
    Base class for a SWMM Simulation.

    The model object provides several options to run a simulation.

    Examples:

    Intialize a simulation and iterate through a simulation. This
    approach requires some clean up.

    >>> from pyswmm import Simulation
    >>>
    >>> sim = Simulation('./TestModel1_weirSetting.inp')
    >>> for step in sim:
    ...     pass
    >>>
    >>> sim.report()
    >>> sim.close()

    Intialize using with statement.  This automatically cleans up
    after a simulation

    >>> from pyswmm import Simulation
    >>>
    >>> with Simulation('./TestModel1_weirSetting.inp') as sim:
    ...     for step in sim:
    ...         pass
    ...     sim.report()

    Initialize the simulation and execute. This style does not allow
    the user to interact with the simulation. However, this approach
    tends to be the fastest.

    >>> from pyswmm import Simulation
    >>>
    >>> sim = Simulation('./TestModel1_weirSetting.inp')
    >>> sim.execute()
    """

    def __init__(self, inputfile, reportfile=None, outputfile=None):
        self._model = PySWMM(inputfile, reportfile, outputfile)
        self._model.swmm_open()
        self._advance_seconds = None
        self._isStarted = False

    def __enter__(self):
        """
        Examples:

        >>> from pyswmm import Simulation
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     for step in sim:
        ...         print(sim.current_time)
        ...     sim.report()
        >>>
        >>> 2015-11-01 14:00:30
        >>> 2015-11-01 14:01:00
        >>> 2015-11-01 14:01:30
        >>> 2015-11-01 14:02:00
        """
        return self

    def __iter__(self):
        """Iterator over Simulation"""
        return self

    def next(self):
        """Next"""
        # Start Simulation
        if not self._isStarted:
            self._model.swmm_start()
            self._isStarted = True

        # Simulation Step Amount
        if self._advance_seconds is None:
            time = self._model.swmm_step()
        else:
            time = self._model.swmm_stride(self._advance_seconds)

        if time <= 0.0:
            self._model.swmm_end()
            raise StopIteration
        return self._model

    def __exit__(self, *a):
        """close"""
        self._model.swmm_end()
        self._model.swmm_close()

    def step_advance(self, advance_seconds):
        """
        Advances the model by X number of seconds instead of
        intervening at every routing step.  This does not change
        the routing step for the simulation; only lets python take
        back control after each advance period.

        :param int advance_seconds: Seconds to Advance simulation

        Examples:

        >>> from pyswmm import Simulation
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     sim.step_advance(300)
        ...     for step in sim:
        ...         print(step.current_time)
        ...         # or here! sim.step_advance(newvalue)
        ...     sim.report()
        >>>
        >>> 2015-11-01 14:00:30
        >>> 2015-11-01 14:01:00
        >>> 2015-11-01 14:01:30
        >>> 2015-11-01 14:02:00
        """
        self._advance_seconds = advance_seconds

    def report(self):
        """
        Writes to report file after simulation

        Examples:

        >>> from pyswmm import Simulation
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     for step in sim:
        ...         pass
        ...     sim.report()
        """
        self._model.swmm_report()

    def close(self):
        """
        Intialize a simulation and iterate through a simulation. This
        approach requires some clean up.

        Examples:

        >>> from pyswmm import Simulation
        >>>
        >>> sim = Simulation('./TestModel1_weirSetting.inp')
        >>> for step in sim:
        ...     pass
        >>>
        >>> sim.report()
        >>> sim.close()
        """
        self.__exit__()

    def execute(self):
        """
        Open an input file, run SWMM, then close the file.

        Examples:

        >>> sim = PYSWMM(r'\\test.inp')
        >>> sim.execute()
        """
        self._model.swmmExec()

    @property
    def start_time(self):
        """Get/set Simulation start time

        Examples:

        >>> from pyswmm import Simulation
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     print sim.start_time
        ...     sim.start_time = datetime(2015,5,10,15,15,1)
        >>>
        >>> datetime.datetime(2015,5,10,15,15,1)
        """
        return self._model.getSimulationDateTime(
            SimulationTime.StartDateTime.value)

    @start_time.setter
    def start_time(self, dtimeval):
        """Set simulation Start time"""
        self._model.setSimulationDateTime(SimulationTime.StartDateTime.value,
                                          dtimeval)

    @property
    def end_time(self):
        """Get/set Simulation end time

        Examples:

        >>> from pyswmm import Simulation
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     print sim.end_time
        ...     sim.end_time = datetime(2016,5,10,15,15,1)
        >>>
        >>> datetime.datetime(2016,5,10,15,15,1)
        """
        return self._model.getSimulationDateTime(
            SimulationTime.EndDateTime.value)

    @end_time.setter
    def end_time(self, dtimeval):
        """Set simulation End time"""
        self._model.setSimulationDateTime(SimulationTime.EndDateTime.value,
                                          dtimeval)

    @property
    def report_start(self):
        """Get/set Simulation report start time

        Examples:

        >>> from pyswmm import Simulation
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     print sim.report_start
        ...     sim.report_start = datetime(2015,5,10,15,15,1)
        >>>
        >>> datetime.datetime(2015,5,10,15,15,1)
        """
        return self._model.getSimulationDateTime(
            SimulationTime.ReportStart.value)

    @report_start.setter
    def report_start(self, dtimeval):
        """Set simulation report start time"""
        self._model.setSimulationDateTime(SimulationTime.ReportStart.value,
                                          dtimeval)

    @property
    def flow_units(self):
        """
        Get Simulation Units (CFS, GPM, MGD, CMS, LPS, MLD).

        :return: Flow Unit
        :rtype: str

        Examples:

        >>> from pyswmm import Simulation
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     print sim.flow_units
        >>>
        >>> CFS
        """
        return self._model.getSimUnit(SimulationUnits.FlowUnits.value)

    @property
    def system_units(self):
        """Get system units (US, SI).

        :return: System Unit
        :rtype: str

        Examples:

        >>> from pyswmm import Simulation
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     print sim.system_units
        >>>
        >>> US
        """
        return self._model.getSimUnit(SimulationUnits.UnitSystem.value)

    @property
    def current_time(self):
        """Get Simulation Current Time.

        :return: Current Simulation Time
        :rtype: Datetime

        Examples:

        >>> from pyswmm import Simulation
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     for step in sim:
        ...         print(sim.current_time)
        ...     sim.report()
        >>>
        >>> 2015-11-01 14:00:30
        >>> 2015-11-01 14:01:00
        >>> 2015-11-01 14:01:30
        >>> 2015-11-01 14:02:00
        """
        return self._model.getCurrentSimualationTime()



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014 Bryant E. McDonnell
#
# Licensed under the terms of the BSD2 License
# See LICENSE.txt for details
# -----------------------------------------------------------------------------

# Local imports
from pyswmm import Link, Links, Simulation
from pyswmm.swmm5 import PySWMM
from pyswmm.tests.data import MODEL_WEIR_SETTING_PATH


def test_links_1():
    with Simulation(MODEL_WEIR_SETTING_PATH) as sim:
        print("\n\n\nLINKS\n")
        c1c2 = Links(sim)["C1:C2"]
        print(c1c2.flow)
        print(c1c2.is_conduit())
        print(c1c2.is_pump())
        print(c1c2.is_orifice())
        print(c1c2.is_weir())
        print(c1c2.is_outlet())
        print(c1c2.connections)
        print(c1c2.inlet_node)
        print(c1c2.outlet_node)
        print(c1c2.average_head_loss)

        sim.step_advance(300)
        for ind, step in enumerate(sim):
            if c1c2.flow > 9.19:
                c1c2.target_setting = 0.9


def test_links_2():
    with Simulation(MODEL_WEIR_SETTING_PATH) as sim:
        for link in Links(sim):
            print(link)
            print(link.linkid)
            print(link.flow_limit)
            link.flow_limit = 10
            print(link.flow_limit)


#def test_links_3():
#    swmmobject = PySWMM(MODEL_WEIR_SETTING_PATH)
#    swmmobject.swmm_open()
#    swmmobject.swmmExec()
#    link = Link(swmmobject, "C2")
#    print(link.flow_limit)
#    swmmobject.swmm_close()
#    print("swmm_step() Check Passed")



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014 Bryant E. McDonnell
#
# Licensed under the terms of the BSD2 License
# See LICENSE.txt for details
# -----------------------------------------------------------------------------
"""Subcatchments module for the pythonic interface to SWMM5."""

# Local imports
from pyswmm.swmm5 import PYSWMMException
from pyswmm.toolkitapi import ObjectType, SubcParams, SubcResults


class Subcatchments(object):
    """
    Subcatchment Iterator Methods.

    :param object model: Open Model Instance

    Examples:

    >>> from pyswmm import Simulation, Subcatchments
    >>>
    >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
    ...     for subcatchment in Subcatchments(sim):
    ...         print subcatchment
    ...         print subcatchment.subcatchmentid
    ...
    >>> <swmm5.Subcatchment object at 0x031B0350>
    >>> S1
    >>> <swmm5.Subcatchment object at 0x030693D0>
    >>> S2
    >>> <swmm5.Subcatchment object at 0x031B0350>
    >>> S3
    >>> <swmm5.Subcatchment object at 0x030693D0>
    >>> S4

    Iterating over Subcatchments Object

    >>> subcatchments = Subcatchments(sim)
    >>> for subcatchment in subcatchments:
    ...     print subcatchment.subcatchmentid
    >>> S0
    >>> S1
    >>> S2
    >>> S3

    Testing Existence

    >>> subcatchments = Subcatchments(sim)
    >>> "S1" in subcatchments
    >>> True

    Initializing a subcatchment Object

    >>> subcatchments = Subcatchments(sim)
    >>> s1 = subcatchments['S1']
    >>> print(s1.area)
    >>> 12
    >>>
    >>> s1.area = 200
    >>> print(s1.area)
    >>> 200
    """

    def __init__(self, model):
        if not model._model.fileLoaded:
            raise PYSWMMException("SWMM Model Not Open")
        self._model = model._model
        self._cuindex = 0
        self._nSubcatchments = self._model.getProjectSize(
            ObjectType.SUBCATCH.value)

    def __len__(self):
        """
        Return number of subcatchments.

        Use the expression 'len(Subcatchments)'.

        :return: Number of Subcatchments
        :rtype: int

        """
        return self._model.getProjectSize(ObjectType.SUBCATCH.value)

    def __contains__(self, subcatchmentid):
        """
        Checks if Subcatchment ID exists.

        :return: ID Exists
        :rtype: bool
        """
        return self._model.ObjectIDexist(ObjectType.SUBCATCH.value,
                                         subcatchmentid)

    def __getitem__(self, subcatchmentid):
        if self.__contains__(subcatchmentid):
            return Subcatchment(self._model, subcatchmentid)
        else:
            raise PYSWMMException("Subcatchment ID Does not Exist")

    def __iter__(self):
        return self

    def next(self):
        if self._cuindex < self._nSubcatchments:
            subcatchmentobject = Subcatchment(self._model,
                                              self._subcatchmentid)
            self._cuindex += 1  # Next Iteration
            return subcatchmentobject
        else:
            raise StopIteration()

    @property
    def _subcatchmentid(self):
        """Subcatchment ID."""
        return self._model.getObjectId(ObjectType.SUBCATCH.value,
                                       self._cuindex)


class Subcatchment(object):
    """
    Subcatchment Methods.

    :param object model: Open Model Instance
    :param str subcatchmentid: Subcatchment ID

    Examples:

    >>> from pyswmm import Simulation, Subcatchments
    >>>
    >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
    ...     s1 = Subcatchments(sim)["S1"]
    ...     print s1.rainfall
    ...     for step in simulation:
    ...         print s1.rainfall
    ... 0.04
    """

    def __init__(self, model, subcatchmentid):
        if not model.fileLoaded:
            raise PYSWMMException("SWMM Model Not Open")
        if subcatchmentid not in model.getObjectIDList(
                ObjectType.SUBCATCH.value):
            raise PYSWMMException("ID Not valid")
        self._model = model
        self._subcatchmentid = subcatchmentid

    # --- Get Parameters
    # -------------------------------------------------------------------------
    @property
    def subcatchmentid(self):
        """
        Get Subcatchment ID.

        :return: Parameter Value
        :rtype: float

        Examples:

        >>> from pyswmm import Simulation, Subcatchments
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     s1 = Subcatchments(sim)["S1"]
        ...     print s1.subcatchmentid
        >>> S1
        """
        return self._subcatchmentid

    @property
    def connection(self):
        """
        Get Subcatchment Outlet Connection.

        This function return the type of loading surface and the ID. The two
        load to objects are nodes and other subcatchments.

        +--------------+---+
        | Node         | 2 |
        +--------------+---+
        | Subcatchment | 1 |
        +--------------+---+

        :return: (Loading Surface Type, ID)
        :rtype: tuple

        Examples:

        >>> from pyswmm import Simulation, Subcatchments
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     s1 = Subcatchments(sim)["S1"]
        ...     print s1.connection
        >>> (2, 'J2')
        """
        return self._model.getSubcatchOutConnection(self._subcatchmentid)

    @property
    def width(self):
        """
        Get/set subcatchment width.

        :return: Parameter Value
        :rtype: float

        Examples:

        >>> from pyswmm import Simulation, Subcatchments
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     s1 = Subcatchments(sim)["S1"]
        ...     print s1.width
        >>> 100.0

        Setting the value

        >>> from pyswmm import Simulation, Subcatchments
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     s1 = Subcatchments(sim)["S1"]
        ...     print s1.width
        ...     s1.width = 30
        ...     print s1.width
        >>> 100
        >>> 30
        """
        return self._model.getSubcatchParam(self._subcatchmentid,
                                            SubcParams.width.value)

    @width.setter
    def width(self, param):
        """Set Subcatchment Width."""
        self._model.setSubcatchParam(self._subcatchmentid,
                                     SubcParams.width.value, param)

    @property
    def area(self):
        """
        Get/set subcatchment area.

        :return: Parameter Value
        :rtype: float

        Examples:

        >>> from pyswmm import Simulation, Subcatchments
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     s1 = Subcatchments(sim)["S1"]
        ...     print s1.area
        >>> 10

        Setting the value

        >>> from pyswmm import Simulation, Subcatchments
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     s1 = Subcatchments(sim)["S1"]
        ...     print s1.area
        ...     s1.area = 50
        ...     print s1.area
        >>> 10
        >>> 50
        """
        return self._model.getSubcatchParam(self._subcatchmentid,
                                            SubcParams.area.value)

    @area.setter
    def area(self, param):
        """Set Subcatchment area."""
        self._model.setSubcatchParam(self._subcatchmentid,
                                     SubcParams.area.value, param)

    @property
    def percent_impervious(self):
        """
        Get/set subcatchment percent impervious.

        :return: Parameter Value
        :rtype: float

        Examples:

        >>> from pyswmm import Simulation, Subcatchments
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     s1 = Subcatchments(sim)["S1"]
        ...     print s1.percent_impervious
        >>> 10

        Setting the value

        >>> from pyswmm import Simulation, Subcatchments
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     s1 = Subcatchments(sim)["S1"]
        ...     print s1.percent_impervious
        ...     s1.percent_impervious = 50
        ...     print s1.percent_impervious
        >>> 10
        >>> 50
        """
        return self._model.getSubcatchParam(self._subcatchmentid,
                                            SubcParams.fracImperv.value)

    @percent_impervious.setter
    def percent_impervious(self, param):
        """Set Subcatchment percent impervious."""
        self._model.setSubcatchParam(self._subcatchmentid,
                                     SubcParams.fracImperv.value, param)

    @property
    def slope(self):
        """
        Get/set subcatchment slope.

        :return: Parameter Value
        :rtype: float

        Examples:

        >>> from pyswmm import Simulation, Subcatchments
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     s1 = Subcatchments(sim)["S1"]
        ...     print s1.slope
        >>> 0.01

        Setting the value

        >>> from pyswmm import Simulation, Subcatchments
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     s1 = Subcatchments(sim)["S1"]
        ...     print s1.slope
        ...     s1.slope = 0.02
        ...     print s1.slope
        >>> 0.1
        >>> 0.2
        """
        return self._model.getSubcatchParam(self._subcatchmentid,
                                            SubcParams.slope.value)

    @slope.setter
    def slope(self, param):
        """Set Subcatchment Ponding Area."""
        self._model.setSubcatchParam(self._subcatchmentid,
                                     SubcParams.slope.value, param)

    @property
    def curb_length(self):
        """
        Get/set subcatchment curb length.

        :return: Parameter Value
        :rtype: float

        Examples:

        >>> from pyswmm import Simulation, Subcatchments
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     s1 = Subcatchments(sim)["S1"]
        ...     print s1.curb_length
        >>> 0

        Setting the value

        >>> from pyswmm import Simulation, Subcatchments
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     s1 = Subcatchments(sim)["S1"]
        ...     print s1.curb_length
        ...     s1.curb_length = 100
        ...     print s1.curb_length
        >>> 0
        >>> 100
        """
        return self._model.getSubcatchParam(self._subcatchmentid,
                                            SubcParams.curbLength.value)

    @curb_length.setter
    def curb_length(self, param):
        """Set Subcatchment curb length."""
        self._model.setSubcatchParam(self._subcatchmentid,
                                     SubcParams.curbLength.value, param)

    @property
    def rainfall(self):
        """
        Get Subcatchment Results for Rainfall.

        If Simulation is not running this method will raise a warning and
        return 0.

        :return: Parameter Value
        :rtype: float

        Examples:

        >>> from pyswmm import Simulation, Subcatchments
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     s1 = Subcatchments(sim)["S1"]
        ...     for step in sim:
        ...         print s1.rainfall
        >>> 0
        >>> 1.2
        >>> 1.5
        >>> 1.9
        >>> 1.2
        """
        return self._model.getSubcatchResult(self._subcatchmentid,
                                             SubcResults.rainfall.value)

    @property
    def evaporation_loss(self):
        """
        Get Subcatchment Results for evaporation loss.

        If Simulation is not running this method will raise a warning and
        return 0.

        :return: Parameter Value
        :rtype: float

        Examples:

        >>> from pyswmm import Simulation, Subcatchments
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     s1 = Subcatchments(sim)["S1"]
        ...     for step in sim:
        ...         print s1.evaporation_loss
        >>> 0.01
        >>> 0.01
        >>> 0.01
        >>> 0.01
        >>> 0.01
        """
        return self._model.getSubcatchResult(self._subcatchmentid,
                                             SubcResults.evapLoss.value)

    @property
    def infiltration_loss(self):
        """
        Get Subcatchment Results for Infiltration Loss.

        If Simulation is not running this method will raise a warning and
        return 0.

        :return: Parameter Value
        :rtype: float

        Examples:

        >>> from pyswmm import Simulation, Subcatchments
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     s1 = Subcatchments(sim)["S1"]
        ...     for step in sim:
        ...         print s1.infiltration_loss
        >>> 0
        >>> 0.01
        >>> 0.01
        >>> 0.01
        >>> 0.01
        """
        return self._model.getSubcatchResult(self._subcatchmentid,
                                             SubcResults.infilLoss.value)

    @property
    def runon(self):
        """
        Get Subcatchment Results for Run On.

        If Simulation is not running this method will raise a warning and
        return 0.

        :return: Parameter Value
        :rtype: float

        Examples:

        >>> from pyswmm import Simulation, Subcatchments
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     s1 = Subcatchments(sim)["S1"]
        ...     for step in sim:
        ...         print s1.runon
        >>> 0
        >>> 1.2
        >>> 1.5
        >>> 1.9
        >>> 1.2
        """
        return self._model.getSubcatchResult(self._subcatchmentid,
                                             SubcResults.runon.value)

    @property
    def runoff(self):
        """
        Get Subcatchment Results for Run Off Rate.

        If Simulation is not running this method will raise a warning and
        return 0.

        :return: Parameter Value
        :rtype: float

        Examples:

        >>> from pyswmm import Simulation, Subcatchments
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     s1 = Subcatchments(sim)["S1"]
        ...     for step in sim:
        ...         print s1.runoff
        >>> 0
        >>> 0
        >>> 0.01
        >>> 0
        >>> 0
        """
        return self._model.getSubcatchResult(self._subcatchmentid,
                                             SubcResults.newRunoff.value)

    @property
    def snow_depth(self):
        """
        Get Subcatchment Results for Snow Depth.

        If Simulation is not running this method will raise a warning and
        return 0.

        :return: Parameter Value
        :rtype: float

        Examples:

        >>> from pyswmm import Simulation, Subcatchments
        >>>
        >>> with Simulation('../test/TestModel1_weirSetting.inp') as sim:
        ...     s1 = Subcatchments(sim)["S1"]
        ...     for step in sim:
        ...         print s1.snow_depth
        >>> 0
        >>> 0.5
        >>> 0.51
        >>> 0.52
        >>> 0.49
        """
        return self._model.getSubcatchResult(self._subcatchmentid,
                                             SubcResults.newSnowDepth.value)



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014 Bryant E. McDonnell
#
# Licensed under the terms of the BSD2 License
# See LICENSE.txt for details
# -----------------------------------------------------------------------------
"""SWMM5 test models."""

# Standard library imports
import os

DATA_PATH = os.path.abspath(os.path.dirname(__file__))

# Test models paths
MODEL_NODE_INFLOWS_PATH = os.path.join(DATA_PATH, 'model_node_inflows.inp')
MODEL_PUMP_SETTINGS_PATH = os.path.join(DATA_PATH, 'model_pump_setting.inp')
MODEL_TOOLKIT_UNITS_PATH = os.path.join(DATA_PATH, 'model_toolkit_units.inp')
MODEL_WEIR_SETTING_PATH = os.path.join(DATA_PATH, 'model_weir_setting.inp')

# Images
IMAGE_NODE_INFLOWS_PATH = os.path.join(DATA_PATH, 'node_inflows.PNG')
IMAGE_WEIR_SETTING_PATH = os.path.join(DATA_PATH, 'weir_setting.PNG')



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014 Bryant E. McDonnell
#
# Licensed under the terms of the BSD2 License
# See LICENSE.txt for details
# -----------------------------------------------------------------------------

# Standard library imports
from random import randint
import sys

# Local imports
from pyswmm import Links, Simulation
from pyswmm.tests.data import MODEL_WEIR_SETTING_PATH


def test_simulation_1():
    sim = Simulation(MODEL_WEIR_SETTING_PATH)
    for ind, step in enumerate(sim):
        print(step.getCurrentSimualationTime())
        sim.step_advance(randint(300, 900))

    sim.report()
    sim.close()


def test_simulation_2():
    with Simulation(MODEL_WEIR_SETTING_PATH) as sim:
        for ind, step in enumerate(sim):
            current_time = step.getCurrentSimualationTime()
            sys.stdout.write("Status: {0}".format(current_time))
            sys.stdout.flush()
            sim.step_advance(randint(300, 900))
        sim.report()


#def test_simulation_3():
#    sim = Simulation(MODEL_WEIR_SETTING_PATH)
#    sim.execute()


def test_simulation_iter():
    with Simulation(MODEL_WEIR_SETTING_PATH) as sim:
        c1c2 = Links(sim)["C1:C2"]
        print(c1c2.flow)

        sim.step_advance(300)
        for ind, step in enumerate(sim):
            print(c1c2.flow, c1c2.target_setting)
            if c1c2.flow > 9.19:
                c1c2.target_setting = 0.9
        sim.report()



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014 Bryant E. McDonnell
#
# Licensed under the terms of the BSD2 License
# See LICENSE.txt for details
# -----------------------------------------------------------------------------
"""SWMM Object Enum."""

# Standard library imports
from enum import Enum


class SimulationTime(Enum):
    StartDateTime = 0
    EndDateTime = 1
    ReportStart = 2


class SimulationUnits(Enum):
    UnitSystem = 0  # (0-US, 1-SI)
    FlowUnits = 1


class SimAnalysisSettings(Enum):
    AllowPonding = 0  # No ponding at nodes
    SkipSteadyState = 1  # Do flow routing in steady state periods
    IgnoreRainfall = 2  # Analyze rainfall/runoff
    IgnoreRDII = 3  # Analyze RDII
    IgnoreSnowmelt = 4  # Analyze snowmelt
    IgnoreGwater = 5  # Analyze groundwater
    IgnoreRouting = 6  # Analyze flow routing
    IgnoreQuality = 7  # Analyze water quality


class SimulationParameters(Enum):
    RouteStep = 0  # Routing time step (sec)
    MinRouteStep = 1  # Minimum variable time step (sec)
    LengtheningStep = 2  # Time step for lengthening (sec)
    StartDryDays = 3  # Antecedent dry days
    CourantFactor = 4  # Courant time step factor
    MinSurfArea = 5  # Minimum nodal surface area
    MinSlope = 6  # Minimum conduit slope
    RunoffError = 7  # Runoff continuity error
    GwaterError = 8  # Groundwater continuity error
    FlowError = 9  # Flow routing error
    QualError = 10  # Quality routing error
    HeadTol = 11  # DW routing head tolerance (ft)
    SysFlowTol = 12  # Tolerance for steady system flow
    LatFlowTol = 13  # Tolerance for steady nodal inflow


class ObjectType(Enum):
    GAGE = 0  # rain gage
    SUBCATCH = 1  # subcatchment
    NODE = 2  # conveyance system node
    LINK = 3  # conveyance system link
    POLLUT = 4  # pollutant
    LANDUSE = 5  # land use category
    TIMEPATTERN = 6  # dry weather flow time pattern
    CURVE = 7  # generic table of values
    TSERIES = 8  # generic time series of values
    CONTROL = 9  # conveyance system control rules
    TRANSECT = 10  # irregular channel cross-section
    AQUIFER = 11  # groundwater aquifer
    UNITHYD = 12  # RDII unit hydrograph
    SNOWMELT = 13  # snowmelt parameter set
    SHAPE = 14  # custom conduit shape
    LID = 15  # LID treatment units
    MAX_OBJ_TYPES = 16  # MaximumObjectTypes


class NodeParams(Enum):
    invertElev = 0  # double
    fullDepth = 1  # double
    surDepth = 2  # double
    pondedArea = 3  # double
    initDepth = 4  # double


class NodeResults(Enum):
    totalinflow = 0  # Total Inflow
    outflow = 1  # Total Outflow
    losses = 2  # Losses (evap + exfiltration loss)
    newVolume = 3  # Current Volume
    overflow = 4  # overflow
    newDepth = 5  # Current water depth
    newHead = 6  # Current water head
    newLatFlow = 7  # Current Lateral Inflow


class NodeType(Enum):
    junction = 0  # Junction Type
    outfall = 1  # Outfall Type
    storage = 2  # Storage Type
    divider = 3  # Divider Type


class LinkParams(Enum):
    offset1 = 0  # double
    offset2 = 1  # double
    q0 = 2  # double
    qLimit = 3  # double
    cLossInlet = 4  # double
    cLossOutlet = 5  # double
    cLossAvg = 6  # double
    seepRate = 7  # double


class LinkResults(Enum):
    newFlow = 0  # double
    newDepth = 1  # double
    newVolume = 2  # double
    surfArea1 = 3  # double
    surfArea2 = 4  # double
    setting = 5  # double
    targetSetting = 6  # double
    froude = 7  # double


class LinkType(Enum):
    conduit = 0  # Conduit Type
    pump = 1  # Pump Type
    orifice = 2  # Orifice Type
    weir = 3  # Weir Type
    outlet = 4  # Outlet Type


class SubcParams(Enum):
    width = 0  # double
    area = 1  # double
    fracImperv = 2  # double
    slope = 3  # double
    curbLength = 4  # double
    # initBuildup = 5  # double


class SubcResults(Enum):
    rainfall = 0  # Current Rainfall
    evapLoss = 1  # Current Evaporation Loss
    infilLoss = 2  # Current Infiltration Loss
    runon = 3  # Subcatchment Runon
    newRunoff = 4  # Current Runoff
    newSnowDepth = 5  # Current Snow Depth


# --- SWMM Output API
# -----------------------------------------------------------------------------
DLLErrorKeys = {
    411: "Input Error 411: no memory allocated for results.",
    412: "Input Error 412: no results; binary file hasn't been opened.",
    421: "Input Error 421: invalid parameter code.",
    434: "File Error  434: unable to open binary output file.",
    435: "File Error  435: run terminated; no results in binary file.",
    441: "Error 441: need to call SMR_open before calling this function"
}


class SMO_elementCount(Enum):
    subcatchCount = 0
    nodeCount = 1
    linkCount = 2
    pollutantCount = 3


class SMO_unit(Enum):
    flow_rate = 0
    concentration = 1


class SMO_apiFunction(Enum):
    getAttribute = 0
    getResult = 1


class SMO_elementType(Enum):
    SM_subcatch = 0
    SM_node = 1
    SM_link = 2
    SM_sys = 3


class SMO_time(Enum):
    reportStep = 0
    numPeriods = 1


class SMO_subcatchAttribute(Enum):
    rainfall_subcatch = 0
    snow_depth_subcatch = 1
    evap_loss = 2
    infil_loss = 3
    runoff_rate = 4
    gwoutflow_rate = 5
    gwtable_elev = 6
    soil_moisture = 7
    pollutant_conc_subcatch = 8


class SMO_nodeAttribute(Enum):
    invert_depth = 0
    hydraulic_head = 1
    stored_ponded_volume = 2
    lateral_inflow = 3
    total_inflow = 4
    flooding_losses = 5
    pollutant_conc_node = 6


class SMO_linkAttribute(Enum):
    flow_rate_link = 0
    flow_depth = 1
    flow_velocity = 2
    flow_volume = 3
    capacity = 4
    pollutant_conc_link = 5


class SMO_systemAttribute(Enum):
    air_temp = 0
    rainfall_system = 1
    snow_depth_system = 2
    evap_infil_loss = 3
    runoff_flow = 4
    dry_weather_inflow = 5
    groundwater_inflow = 6
    RDII_inflow = 7
    direct_inflow = 8
    total_lateral_inflow = 9
    flood_losses = 10
    outfall_flows = 11
    volume_stored = 12
    evap_rate = 13



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014 Bryant E. McDonnell
#
# Licensed under the terms of the BSD2 License
# See LICENSE.txt for details
# -----------------------------------------------------------------------------

# Local imports
from pyswmm import Node, Nodes, Simulation
from pyswmm.swmm5 import PySWMM
from pyswmm.tests.data import MODEL_WEIR_SETTING_PATH
from pyswmm.utils.fixtures import get_model_files
import pyswmm.toolkitapi as tka


def test_nodes_1():
    swmmobject = PySWMM(MODEL_WEIR_SETTING_PATH)
    swmmobject.swmm_open()
    node = Node(swmmobject, "J1")
    print(node.invert_elevation)


def test_nodes_2():
    with Simulation(MODEL_WEIR_SETTING_PATH) as sim:
        print("\n\n\nNODES\n")
        for node in Nodes(sim):
            print(node)
            print(node.nodeid)
            print(node.invert_elevation)
            node.invert_elevation = 10
            print(node.invert_elevation)


def test_nodes_3():
    with Simulation(MODEL_WEIR_SETTING_PATH) as sim:
        print("\n\n\nNODES\n")
        j1 = Nodes(sim)["J1"]
        print(j1.is_divider())
        print(j1.is_junction())
        print(j1.is_outfall())
        print(j1.is_storage())
        print(j1.invert_elevation)


def test_nodes_4():
    swmmobject = PySWMM(*get_model_files(MODEL_WEIR_SETTING_PATH))
    swmmobject.swmm_open()

    print(swmmobject.getNodeParam('J2', tka.NodeParams.invertElev))
    swmmobject.setNodeParam('J2', tka.NodeParams.invertElev, 19)
    print(swmmobject.getNodeParam('J2', tka.NodeParams.invertElev))

    print(swmmobject.getLinkParam('C1:C2', tka.LinkParams.offset1))
    swmmobject.setLinkParam('C1:C2', tka.LinkParams.offset1, 19)
    print(swmmobject.getLinkParam('C1:C2', tka.LinkParams.offset1))

    print(swmmobject.getSubcatchParam('S2', tka.SubcParams.area))
    swmmobject.setSubcatchParam('S2', tka.SubcParams.area, 19)
    print(swmmobject.getSubcatchParam('S2', tka.SubcParams.area))

    swmmobject.swmm_start()
    i = 0
    while (True):
        time = swmmobject.swmm_stride(600)
        i += 1

        if (time <= 0.0):
            break

        if i % 144 == 0:
            print(i)

        swmmobject.setLinkParam('C1:C2', tka.LinkParams.qLimit, 0.25)

    swmmobject.swmm_end()
    swmmobject.swmm_close()
    print("swmm_step() Check Passed")



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014 Bryant E. McDonnell
#
# Licensed under the terms of the BSD2 License
# See LICENSE.txt for details
# -----------------------------------------------------------------------------

# Local imports
from pyswmm import Simulation, Subcatchment, Subcatchments
from pyswmm.swmm5 import PySWMM
from pyswmm.tests.data import MODEL_WEIR_SETTING_PATH


def test_subcatchments_1():
    with Simulation(MODEL_WEIR_SETTING_PATH) as sim:
        print("\n\n\nSubcatchments\n")
        S3 = Subcatchments(sim)["S3"]
        print(S3.area)
        print(S3.width)
        print(S3.connection)
        print(S3.percent_impervious)
        print(S3.slope)

        sim.step_advance(300)
        for ind, step in enumerate(sim):
            print(S3.runoff)


def test_subcatchments_2():
    with Simulation(MODEL_WEIR_SETTING_PATH) as sim:
        for subcatchment in Subcatchments(sim):
            print(subcatchment)
            print(subcatchment.subcatchmentid)
            print(subcatchment.curb_length)
            subcatchment.curb_length = 10
            print(subcatchment.curb_length)



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014 Bryant E. McDonnell
#
# Licensed under the terms of the BSD2 License
# See LICENSE.txt for details
# -----------------------------------------------------------------------------

# Standard library imports
from datetime import timedelta

# Local imports
from pyswmm.swmm5 import PySWMM
from pyswmm.tests.data import MODEL_WEIR_SETTING_PATH
from pyswmm.utils.fixtures import get_model_files


def test_current_time():
    swmmobject = PySWMM(*get_model_files(MODEL_WEIR_SETTING_PATH))
    swmmobject.swmm_open()
    date = swmmobject.getSimulationDateTime(0)
    print(date)
    swmmobject.swmm_start()

    i = 0
    while (True):
        time = swmmobject.swmm_stride(600)
        print(time)
        i += 1
        current_time = swmmobject.getCurrentSimualationTime()
        print(current_time)
        print(type(swmmobject.getCurrentSimualationTime()))

        if (time <= 0.0):
            break
        if i % 144 == 0:
            print(i)

    swmmobject.swmm_end()
    swmmobject.swmm_report()
    swmmobject.swmm_close()


def test_set_current_time():
    swmmobject = PySWMM(*get_model_files(MODEL_WEIR_SETTING_PATH))
    swmmobject.swmm_open()

    print(swmmobject.getSimulationDateTime(0))
    print(swmmobject.getSimulationDateTime(0) - timedelta(days=10))
    swmmobject.setSimulationDateTime(
        0,
        swmmobject.getSimulationDateTime(0) - timedelta(days=10), )
    print(swmmobject.getSimulationDateTime(0))

    print(swmmobject.getSimulationDateTime(1))
    print(swmmobject.getSimulationDateTime(1) - timedelta(days=10))
    swmmobject.setSimulationDateTime(
        1,
        swmmobject.getSimulationDateTime(1) - timedelta(days=10), )
    print(swmmobject.getSimulationDateTime(1))

    print(swmmobject.getSimulationDateTime(2))
    print(swmmobject.getSimulationDateTime(2) - timedelta(days=10))
    swmmobject.setSimulationDateTime(
        2,
        swmmobject.getSimulationDateTime(2) - timedelta(days=10), )
    print(swmmobject.getSimulationDateTime(2))

    swmmobject.swmm_start()

    i = 0
    while (True):
        time = swmmobject.swmm_stride(600)
        i += 1
        print(swmmobject.getCurrentSimualationTime())
        print(type(swmmobject.getCurrentSimualationTime()))

        if (time <= 0.0):
            break

        if i % 144 == 0:
            print(i)

    swmmobject.swmm_end()
    swmmobject.swmm_close()



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014 Bryant E. McDonnell
#
# Licensed under the terms of the BSD2 License
# See LICENSE.txt for details
# -----------------------------------------------------------------------------
"""Python setup.py installer script."""

# Standard library imports
import ast
import os
import sys

# Third party imports
from setuptools import find_packages, setup

HERE = os.path.abspath(os.path.dirname(__file__))
PY2 = sys.version_info.major == 2


def get_version(module='pyswmm'):
    """Get version."""
    with open(os.path.join(HERE, module, '__init__.py'), 'r') as f:
        data = f.read()
    lines = data.split('\n')
    for line in lines:
        if line.startswith('VERSION_INFO'):
            version_tuple = ast.literal_eval(line.split('=')[-1].strip())
            version = '.'.join(map(str, version_tuple))
            break
    return version


def get_description():
    """Get long description."""
    with open(os.path.join(HERE, 'README.rst'), 'r') as f:
        data = f.read()
    return data


REQUIREMENTS = []

if sys.version_info < (3, 4):
    REQUIREMENTS.append('enum34')

setup(
    name='pyswmm',
    version=get_version(),
    description='Python Wrapper for SWMM5 API',
    long_description=get_description(),
    url='https://github.com/OpenWaterAnalytics/pyswmm/wiki',
    author='Bryant E. McDonnell (EmNet LLC)',
    author_email='bemcdonnell@gmail.com',
    install_requires=REQUIREMENTS,
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    package_data={'': ['lib/windows/swmm5.dll', 'LICENSE.txt']},
    include_package_data=True,
    license="BSD2 License",
    keywords="swmm5, swmm, hydraulics, hydrology, modeling, collection system",
    classifiers=[
        "Topic :: Scientific/Engineering",
        "Topic :: Documentation :: Sphinx",
        "Operating System :: Microsoft :: Windows",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: C",
        "Development Status :: 4 - Beta",
    ])



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014 Bryant E. McDonnell
#
# Licensed under the terms of the BSD2 License
# See LICENSE.txt for details
# -----------------------------------------------------------------------------
"""SWMM5 compiled libraries."""

# Standard library imports
import os

HERE = os.path.abspath(os.path.dirname(__file__))

# Library paths
LIB_SWMM_WIN_32 = os.path.join(HERE, 'windows', 'swmm5.dll').replace('\\', '/')



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014 Bryant E. McDonnell
#
# Licensed under the terms of the BSD2 License
# See LICENSE.txt for details
# -----------------------------------------------------------------------------

# Standard library imports
import os
import random

# Third party imports
from matplotlib._png import read_png
from matplotlib.offsetbox import AnnotationBbox, OffsetImage
import matplotlib.animation as animation
import matplotlib.pyplot as plt

# Local imports
from pyswmm.swmm5 import PySWMM
from pyswmm.tests.data import (DATA_PATH, IMAGE_NODE_INFLOWS_PATH,
                               MODEL_NODE_INFLOWS_PATH)
from pyswmm.utils.fixtures import get_model_files
import pyswmm.toolkitapi as tka


def test_plot_node_inflows():
    swmmobject = PySWMM(*get_model_files(MODEL_NODE_INFLOWS_PATH))
    swmmobject.swmm_open()
    swmmobject.swmm_start()

    fig = plt.figure()
    ax = fig.add_subplot(2, 3, (1, 2))
    ax.set_ylabel('Flow Rate')
    line, = ax.plot([], [], label='C3')
    ax.grid()
    ax.legend()

    ax2 = fig.add_subplot(2, 3, (4, 5), sharex=ax)
    ax2.set_ylabel('Toolkit API\nInflow')
    line2, = ax2.plot([], [], label='J1')
    ax2.grid()

    xdata, ydata = [], []
    ydata2 = []

    ax3 = fig.add_subplot(2, 3, (3, 6))

    arr_lena = read_png(IMAGE_NODE_INFLOWS_PATH)
    imagebox = OffsetImage(arr_lena, zoom=0.67)
    ab = AnnotationBbox(
        imagebox,
        (0.5, 0.5),
        xybox=(0.5, 0.5),
        xycoords='data',
        boxcoords="offset points",
        pad=0.0, )

    ax3.add_artist(ab)
    ax3.axis('off')

    def data_gen(t=0):
        i = 0
        while (True):
            if i >= 80:
                swmmobject.setNodeInflow('J1', random.randint(1, 5))
            time = swmmobject.swmm_stride(1500)
            i += 1

            if i > 0 and time == 0.0:
                break
            if i > 0 and time > 0:
                yield time

    def run(t):
        xdata.append(t)
        new_y = swmmobject.getLinkResult('C2', tka.LinkResults.newFlow)
        ydata.append(new_y)

        new_y2 = swmmobject.getNodeResult('J1', tka.NodeResults.newLatFlow)
        ydata2.append(new_y2)

        xmin, xmax = ax.get_xlim()
        ymin, ymax = ax.get_ylim()
        ymin2, ymax2 = ax2.get_ylim()

        # ax
        if new_y > ymax:
            ax.set_ylim(-0.1, 1.5 * ymax)
        if t >= xmax:
            ax.set_xlim(xmin, 1.5 * xmax)
            ax.figure.canvas.draw()
        line.set_data(xdata, ydata)

        # ax1
        if new_y2 > ymax2:
            ax2.set_ylim(-0.1, 1.2 * ymax2)
        line2.set_data(xdata, ydata2)

    ani = animation.FuncAnimation(
        fig,
        run,
        data_gen,
        blit=False,
        repeat=False,
        save_count=800,
        interval=10, )

    show_figure = False
    if show_figure:
        plt.show()
    else:
        movie_path = os.path.join(DATA_PATH, "node_inflows.mp4")
        print(movie_path, ani)
        # ani.save(movie_path, fps=20, dpi=170, bitrate=50000)
    plt.close()

    swmmobject.swmm_end()
    swmmobject.swmm_report()
    swmmobject.swmm_close()
    print("Check Passed")



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014 Bryant E. McDonnell
#
# Licensed under the terms of the BSD2 License
# See LICENSE.txt for details
# -----------------------------------------------------------------------------
"""Python Wrapper for Stormwater Management Model (SWMM5)."""

# Local imports
from pyswmm.links import Link, Links
from pyswmm.nodes import Node, Nodes
from pyswmm.simulation import Simulation
from pyswmm.subcatchments import Subcatchment, Subcatchments

VERSION_INFO = (0, 3)
__version__ = '.'.join(map(str, VERSION_INFO))
__author__ = 'Bryant E. McDonnell (EmNet LLC) - bemcdonnell@gmail.com'
__copyright__ = 'Copyright (c) 2016 Bryant E. McDonnell'
__licence__ = 'BSD2'
__all__ = [Link, Links, Node, Nodes, Subcatchment, Subcatchments, Simulation]
